"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var importReg = /^import\s+((\* as )?\s*([\w_]+)\s*)?,?\s*(\{\s*([\w_,\s]*)\s\})?\s*(from\s+)?(['"][\._\-\w\/]+['"])/gm;
var exportRef = /^export\s+(default)?\s*(const|let|var|class|function|interface|enum|interface)?\s*([\w_]*)/gm;
function transform(source, proxyFunction) {
    importReg.lastIndex = 0;
    var moduleExports = [];
    var defaultExport = null;
    exportRef.lastIndex = 0;
    var match = null;
    while ((match = exportRef.exec(source)) != null) {
        if (match[2] == 'enum' || match[2] == 'interface')
            continue;
        if (match[1]) {
            if (defaultExport) {
                console.error('Multiple default exports!');
                throw new Error('Multiple default exports');
            }
            else {
                defaultExport = match[3];
            }
        }
        else {
            moduleExports.push(match[3]);
        }
    }
    source = source.replace(/^export\s*\{?\s+(default)?\s*\}?(function\s*\(|\{|\()?\s*(from\s+([\W\w'_]+)\s*;)?/gm, function (full, defaultKw, type, fromGroup, fromPath) {
        console.log(full);
        console.log(type);
        console.log(fromGroup);
        console.log(fromPath);
        if (fromPath) {
            defaultExport = 'defaultObject';
            return "const defaultObject = require(" + fromPath + ");";
        }
        else if (type) {
            if (type.match(/function(\s|\()/)) {
                defaultExport = 'defaultFunction';
                return 'function defaultFunction(';
            }
            else if (type.match(/\{/)) {
                defaultExport = 'defaultObject';
                return 'const defaultObject = {';
            }
            else if (type.match(/\(/)) {
                defaultExport = 'defaultFunction';
                return 'const defaultFunction = (';
            }
        }
        return '';
    });
    var allExports = '';
    if (moduleExports.length) {
        moduleExports.forEach(function (e) {
            if (e.trim()) {
                allExports += "\nexports." + e + " = " + e + ";";
            }
        });
    }
    if (defaultExport) {
        allExports += "\nexports.default = " + defaultExport + ";";
    }
    source += allExports;
    return source.replace(importReg, function (wholeMatch, defaultImport, defaultAs, defaultAsVar, specificImportWrapper, specificImports, fromKeyword, source) {
        if (!fromKeyword) {
            return createProxy(proxyFunction, source);
        }
        else if (defaultImport && specificImports) {
            return (handleDefaultImports(defaultAs, defaultAsVar, defaultImport, source, proxyFunction) +
                '\n' +
                handleSpecificImports(specificImports, source, proxyFunction));
        }
        else if (defaultImport) {
            return handleDefaultImports(defaultAs, defaultAsVar, defaultImport, source, proxyFunction);
        }
        else if (specificImports) {
            return handleSpecificImports(specificImports, source, proxyFunction);
        }
        return wholeMatch;
    });
}
exports.default = transform;
function createProxy(proxyFunction, source) {
    return proxyFunction
        ? proxyFunction + " (() => require(" + source + "), " + source + ")"
        : "require(" + source + ")";
}
function handleSpecificImports(specificImports, source, proxyFunction) {
    specificImports = specificImports.replace(/ as /g, ': ');
    return "const { " + specificImports + " } = " + createProxy(proxyFunction, source);
}
function handleDefaultImports(defaultAs, defaultAsVar, defaultImport, source, proxyFunction) {
    if (defaultAs) {
        return "const " + defaultAsVar.trim() + " = " + createProxy(proxyFunction, source);
    }
    else {
        return "const " + defaultImport.trim() + " = " + createProxy(proxyFunction, source) + ".default || " + createProxy(proxyFunction, source);
    }
}
